;***************************************************************************
;*   Library Name: Robots
;*       Coded By: John Logsdon
;*   Last Updated: 8/5/2012
;*        Purpose: Handles all of the robot procedures
;***************************************************************************


;*************************************************************
; PROCEDURE: LoadRobotFrames(...)
;
; This function moves the player's character by a certain
; number of pixels.
;
; Arguments: 
;   Robot_Full_Path: File name, including full path
;   RobotWidth:  How wide the image is
;   RobotHeight: how high the image is
;   RobotSpacer: how many pixels between each image in the file
;
; Returns:
;   0 = Everything loaded fine
;   ? = Load failed
;*************************************************************
Procedure LoadRobotFrames(Robot_Full_Path.s,RobotWidth.l,RobotHeight.l,RobotSpacer.l)

  Global Dim RobotImages.Robot_Frame_Images(0)
  
  ; first off let's load the full image containing all the tiles
  ; into a temporary space
  Temp_Image = LoadSprite(#PB_Any,Robot_Full_Path.s)
  
  ImageSizeX.l = SpriteWidth(Temp_Image)
  ImageSizeY.l = SpriteHeight(Temp_Image)
  
  RobotColumns.l = ImageSizeX.l / RobotWidth.l
  RobotRows.l = ImageSizeY.l / RobotHeight.l
  
  Global Dim RobotImages.Robot_Frame_Images(RobotRows.l * RobotColumns.l)
  
  If Temp_Image = 0
    ProcedureReturn(-1)
  EndIf
  
  ; get the current buffer so we can restore to it
  DisplaySprite(Temp_Image,0,0)
  
  ; setup our basic vars.  The X and Y values will be whatever the XSpacer
  ; and YSpacer values are. If they are 0, then it's assumed that there is
  ; no space between the tiles. If you look at the sample images included
  ; with this demo you'll see a white box around each image...that's why
  ; I have these spacers...we don't wanna load the boxes, just the images.
  X.l = RobotSpacer.l
  Y.l = RobotSpacer.l
  
  ; keep track of the number of images
  ImageNumber.l = 0
  
  ; run through the total number of rows (minus 1, of course)
  For Rows.l = 0 To RobotRows.l -1
    ; and run through all the columns per row (minus 1 again)
    For Columns.l = 0 To RobotColumns.l -1
      ; create a new TileList element and assign it the current
      ; ImageNumber.  Then populate the TileList element with
      ; the TileWidth, TileHeight, and the actual Tile_Image
      RobotImages(ImageNumber)\Width = RobotWidth
      RobotImages(ImageNumber)\Height = RobotHeight
      
      ; then we grab the image based off the size setup in CreateImage(...)
      ; from our X,Y location
      RobotImages(ImageNumber)\Image=GrabSprite(#PB_Any,X,Y,RobotWidth,RobotHeight)
      
      ; now we add X to the TileWidth and the XSpacer to get the new X position.
      ; So, if our current X = 2, and the TileWidth = 32 and the XSpacer = 2, we'd
      ; have X = 2 + 32 + 2, or 36.  This means that the next time we call         
      ; GrabImage(...) it will start grabbing from the X position 36 (or the 36th pixel 
      ; from the left).         
      X.l = X.l + RobotWidth.l + RobotSpacer.l
      
      ; increment our tile counter (for the array positioning)
      ImageNumber.l = ImageNumber.l + 1
    Next
    ; we've finished with that row, so reset X back to the spacer position
    X.l = RobotSpacer.l
    
    ; now we add Y to the TileHeight and the YSpacer to get the new Y position.
    ; So, if our current Y = 2, and the TileHeight = 32 and the YSpacer = 2, we'd
    ; have Y = 2 + 32 + 2, or 36.  This means that the next time we call  
    ; GrabImage(...) it will start grabbing from the Y position 36 (or the 36th pixel 
    ; from the top).         
    Y.l = Y.l + RobotHeight.l + RobotSpacer.l
  Next
  
  ; free the image from memory, so we don't hold memory for no reason
  FreeSprite(Temp_Image)
  
  ProcedureReturn(0)
EndProcedure


;*************************************************************
; PROCEDURE: AddRobot(...)
;
; This function adds a robot to the Robot() list.
;
; Arguments: 
;   X: X position (array position, not pixel)
;   Y: Y position (array position, not pixel)
;   MinX: farthest X left (array position, not pixel)
;   MaxX: farthest X right (array position, not pixel)
;   TileWidth: how wide are the images
;   TileHeight: how high are the images
;
; Returns:
;   N/A
;*************************************************************
Procedure AddRobot(X.l,Y.l,MaxX.l,MaxY.l,TileWidth,TileHeight)
  X = X * TileWidth
  Y = Y * TileHeight
  MaxX = MaxX * TileWidth
  MaxY = MaxY * TileHeight
  ; Add it to the ladder Type
  AddElement(Robot())
  Robot()\WorldX = X
  Robot()\WorldY = Y
  Robot()\ScreenX = X
  Robot()\ScreenY = Y
  Robot()\MinimumX = MaxX
  Robot()\MaximumX = MaxY
  Robot()\Direction = Random(3) + 1
  Robot()\Frame = 0
  Robot()\Hit = 0
  Robot()\Dead = 0
  Robot()\Armor = 25
  Robot()\Timer = ElapsedMilliseconds()
  Robot()\Timeout = 50
  Robot()\ActionTimer = ElapsedMilliseconds()
  Robot()\ActionTimeout = Random(4000) + 500
  Robot()\Firing = 0
EndProcedure


;*************************************************************
; PROCEDURE: MoveRobots(...)
;
; This function moves robots around, does a little random
; decision making, and makes firing decisions.
;
; Arguments: 
;   Current_Time: last call to elapsedmilliseconds()
;   MoveMap: Is the map moving?
;   Direction: What direction is it moving?
;   TileWidth: how wide are the tiles
;   TileHeight: how high are the tiles
;
; Returns:
;   N/A
;*************************************************************
Procedure MoveRobots(Current_Time,MoveMap.l,Direction.l,TileWidth,TileHeight)
  ; Go through the robots
  ForEach(Robot())
     ; if it's time for a robot to make a decision (action), let's do it
     If Current_Time > Robot()\ActionTimer + Robot()\ActionTimeout
        ; if the robot is on the same line as the player, the robot's not been hit, the player's not dead, and the level's not in the startup phase
        If Robot()\WorldY >= PlayerWorldY And Robot()\WorldY <= PlayerWorldY + 25 And Robot()\Hit = 0 And PlayerDead = 0 And StartingLevel = 0
           ; use a temp var to flag if the robot is close or not
           Robot_Close = 0
           ; if the player is with 300 units left of the robot, fire at the player
           If Robot()\WorldX > PlayerWorldX And Robot()\WorldX - PlayerWorldX < 300
              ; set the direction to face the player
              Robot()\Direction = 1
              ; set the firing flag to start the firing
              Robot()\Firing = 1
              ; set the appropriate firing frame
              Robot()\Frame = 23
              ; reset the action timer
              Robot()\ActionTimer = Current_Time
              Robot()\ActionTimeout = 500
              ; mark that the robot is indeed close
              Robot_Close = 1
           EndIf
           ; do the same thing as above if the player is to the right of the robot, but still within 300 units
           If Robot()\WorldX < PlayerWorldX And PlayerWorldX - Robot()\WorldX < 300
              Robot()\Direction = 2
              Robot()\Firing = 1
              Robot()\Frame = 40
              Robot()\ActionTimer = Current_Time
              Robot()\ActionTimeout = 500
              Robot_Close = 1
           EndIf
           ; if the robot isn't close to the player
           If Robot_Close = 0
              ; pick a direction and go...this could be "stand still" or walking...this isn't an overly smart robot, ya know.
              Robot()\Direction = Random(3) + 1
              Robot()\ActionTimer = Current_Time
              Robot()\ActionTimeout = Random(4000) + 500
           EndIf
        Else ; if we're not on the same line as the player and all that
           ; pick a random direction and go      
           Robot()\Direction = Random(3) + 1
           Robot()\ActionTimer = Current_Time
           Robot()\ActionTimeout = Random(4000) + 500
        EndIf
     EndIf

     ; now let's animate the robot, if it's the proper time
     If Current_Time > Robot()\Timer + Robot()\Timeout
        ; if the robot is firing
        If Robot()\Firing = 1 
           ; see which way he's facing
           Select Robot()\Direction
             Case 1
                ; and update the frames accordingly
                Robot()\Frame = Robot()\Frame - 1
                If Robot()\Frame < 16
                   Robot()\Frame = 7
                   Robot()\Firing = 0
                   Robot()\ActionTimer = Current_Time
                   Robot()\ActionTimeout = 500
                EndIf
                ; when the robot hits the middle frame, add in the laser
                If Robot()\Frame = 19
                   AddLaser(1,Robot()\ScreenX,Robot()\ScreenY+11,Robot()\WorldX,Robot()\WorldY,1,#TileWidth,#TileHeight)
                EndIf
             Case 2
                Robot()\Frame = Robot()\Frame + 1
                If Robot()\Frame > 47
                   Robot()\Frame = 24
                   Robot()\Firing = 0
                   Robot()\ActionTimer = Current_Time
                   Robot()\ActionTimeout = 500
                EndIf
                ; when the robot hits the middle frame, add in the laser
                If Robot()\Frame = 44
                   AddLaser(1,Robot()\ScreenX+32,Robot()\ScreenY+11,Robot()\WorldX+32,Robot()\WorldY,2,#TileWidth,#TileHeight)
                EndIf
           EndSelect
           ; reset the timer
           Robot()\Timer = Current_Time
        Else ; if the robot is not firing
           ; see which was he's facing and if he's still or moving
           Select Robot()\Direction
              Case 1   ; still, facing left
                     ; if he's been hit or is dead
                     If Robot()\Hit > 0 Or Robot()\Dead > 0
                        ; if dead
                        If Robot()\Dead > 0
                           ; play the death animation until he's completely bye bye
                           Robot()\Frame = Robot()\Frame - 1
                           If Robot()\Frame < 52
                              Robot()\Dead = -1
                           EndIf
                           ; otherwise, just show the hit
                        Else
                           Robot()\Frame = 55
                           Robot()\Hit = Robot()\Hit - 1
                        EndIf
                     Else ; if he's neither hit nor dead, just go by updating the frames as usual
                        Robot()\Frame = Robot()\Frame - 1
                        If Robot()\Frame < 0
                           Robot()\Frame = 7
                        EndIf
                        If Robot()\Frame > 7
                           Robot()\Frame = 0
                        EndIf
                     EndIf
              ;; follow the comments in Case 1
              Case 2   ; still, facing right
                     If Robot()\Hit > 0 Or Robot()\Dead > 0
                        If Robot()\Dead > 0
                           Robot()\Frame = Robot()\Frame + 1
                           If Robot()\Frame > 51
                              Robot()\Dead = -1
                           EndIf
                        Else
                        Robot()\Frame = 48
                           Robot()\Hit = Robot()\Hit - 1
                        EndIf
                     Else
                        Robot()\Frame = Robot()\Frame + 1
                        If Robot()\Frame < 24
                           Robot()\Frame = 24
                        EndIf
                        If Robot()\Frame > 31
                           Robot()\Frame = 24
                        EndIf
                     EndIf

              ;; follow the comments in Case 1
              Case 3   ; Left
                     If Robot()\Hit > 0 Or Robot()\Dead > 0
                        If Robot()\Dead > 0
                           Robot()\Frame = Robot()\Frame - 1
                           If Robot()\Frame < 52
                              Robot()\Dead = -1
                           EndIf
                        Else
                           Robot()\Frame = 55
                           Robot()\Hit = Robot()\Hit - 1
                        EndIf
                     Else
                        Robot()\Frame = Robot()\Frame - 1
                        If Robot()\Frame < 9
                           Robot()\Frame = 15
                        EndIf
                        If Robot()\Frame > 15
                           Robot()\Frame = 15
                        EndIf
                        OldX = Robot()\WorldX
                        Robot()\WorldX = Robot()\WorldX - #MoveDistance
                        If Robot()\WorldX < Robot()\MinimumX
                           Robot()\WorldX = OldX
                           Robot()\Direction = 1
                        Else
                           Robot()\ScreenX = Robot()\ScreenX - #MoveDistance
                        EndIf
                     EndIf

              ;; follow the comments in Case 1
              Case 4   ; Right
                     If Robot()\Hit > 0 Or Robot()\Dead > 0
                        If Robot()\Dead > 0
                           Robot()\Frame = Robot()\Frame + 1
                           If Robot()\Frame > 51
                              Robot()\Dead = -1
                           EndIf
                        Else
                           Robot()\Frame = 48
                           Robot()\Hit = Robot()\Hit - 1
                        EndIf
                     Else
                        Robot()\Frame = Robot()\Frame + 1
                        If Robot()\Frame < 32
                           Robot()\Frame = 32
                        EndIf
                        If Robot()\Frame > 39
                           Robot()\Frame = 32
                        EndIf
                        OldX = Robot()\WorldX
                        Robot()\WorldX = Robot()\WorldX + #MoveDistance
                        If Robot()\WorldX > Robot()\MaximumX
                           Robot()\WorldX = OldX
                           Robot()\Direction = 2
                        Else
                           Robot()\ScreenX = Robot()\ScreenX + #MoveDistance
                        EndIf
                     EndIf
           EndSelect
        EndIf
        ; reset the timer
        Robot()\Timer = Current_Time
     EndIf

     ; if the map is moving, make sure to update the robot by the distance the map moves
     ; according to the player's direction
     If MoveMap.l = 1
        Select Direction.l 
           Case 1
               Robot()\ScreenY = Robot()\ScreenY + #MoveDistance
           Case 2
               Robot()\ScreenY = Robot()\ScreenY - #MoveDistance
           Case 3
               Robot()\ScreenX = Robot()\ScreenX + #MoveDistance
           Case 4
               Robot()\ScreenX = Robot()\ScreenX - #MoveDistance
        EndSelect
     EndIf

     ; setup a couple of temp vars to help with typing so much
     RobotX = Robot()\ScreenX
     RobotY = Robot()\ScreenY
     ; show the robot
     DisplayTransparentSprite(RobotImages(Robot()\Frame)\Image,RobotX,RobotY)
     ; put up the healthbar for the robot based on its armor strength, using
     ; color schemes.
     If StartDrawing(ScreenOutput())
        DrawingMode(4)
        Box(RobotX,RobotY-5,32,3,RGB(255,255,255))
        DrawingMode(0)
        Box(RobotX+1,RobotY-4,30,1,RGB(155,155,155))
        If Robot()\Armor >= 20
           Box(RobotX+1,RobotY-4,Robot()\Armor+5,1,RGB(0,255,0))
        EndIf
        If Robot()\Armor >= 10 And Robot()\Armor < 20
           Box(RobotX+1,RobotY-4,Robot()\Armor+2,1,RGB(255,255,0))
        EndIf
        If Robot()\Armor < 10
           Box(RobotX+1,RobotY-4,Robot()\Armor,1,RGB(255,0,0))
        EndIf
     EndIf
    StopDrawing()

    ; if the robot is scheduled for the trash-compactor...
    If Robot()\Dead = -1
       ; crush it
       DeleteElement(Robot())
    EndIf
  Next
EndProcedure

;*************************************************************
; PROCEDURE: DeleteRobots(...)
;
; Nukes all the robots from the list
;
;*************************************************************
Procedure DeleteRobots()
  ForEach(Robot())
     DeleteElement(Robot())
   Next
EndProcedure 
; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 410
; FirstLine = 372
; Folding = -